#version 120
varying vec3 plane_offset;
varying vec3 plane_normal;
varying float dist;
varying vec4 col;
varying float radius;
varying float structural_abstraction;
varying float illustrativeness;

varying float hint;
varying float ds;

varying float type;

// Pastel colors
varying vec3 pastel_light;
varying vec3 pastel_dark;

varying float ny;
varying float nz;
varying vec3 normal_1;
varying vec3 normal_2;
varying vec3 normalS;
varying vec3 normalP;

// Ambient occlusion
uniform float ao_strip_width;
uniform sampler2D ambient_occlusion0;
uniform sampler1D ao_func;
varying float ao_contrib;
uniform float depth_cueing;
uniform float ambLight;

// Debug vars
varying vec3 direct;

vec4 color(float r2) {
	float gradient;

	if (structural_abstraction < 0.6) {
		gradient = mix(r2, 1.0, structural_abstraction / 0.6);
		return col * gradient;
	}
	else {
		return col;
	}
}

vec4 pastel(float r2) {
	float gradient;
	
	gradient = floor(r2 + 0.7);
	return vec4(mix(pastel_dark, pastel_light, gradient), 1.0);
}

vec4 grey(float r2) {
	vec4 grey;
	vec4 pas;
	float tmp;
	float gradient;

	pas = pastel(r2);
	tmp = (0.30 * col.r + 0.59 * col.g + 0.11 * col.b);
	grey = vec4(tmp);

	if (structural_abstraction < 0.6) {
		gradient = mix(r2, 1.0, structural_abstraction / 0.6);
		return grey * gradient;
	}
	else {
		return grey;
	}
}

vec4 hatch(float r2) {
  return vec4(0.0);
}

vec4 get_frag_color(float r2)
{
	float mx;
	vec4 colorv;

	if (illustrativeness <= (1.0 / 3.0)) {
		mx = 3.0 * illustrativeness;
		colorv = mix(color(r2), pastel(r2), mx);
	}
	else if (illustrativeness <= (2.0 / 3.0)) {
		mx = 3.0 * (illustrativeness - (1.0 / 3.0));
		colorv = mix(pastel(r2), grey(r2), mx);		
	}
	else {
		mx = 3.0 * (illustrativeness - (2.0 / 3.0));
		colorv = mix(grey(r2), hatch(r2), mx);
	}
	colorv.a = 1.0;
	return colorv;
}

void main(void)
{
  vec4 pos;
  float r2;
  vec3 normal;

	vec4 local_normal;
  vec4 global_normal_tmp;
  vec3 global_normal;

	vec3 univpos;
	vec2 texpos;
	vec4 ao_col;
	vec4 colorv;

	float r, s;

	if(type < 0.5) {
	  // Drawing a cylinder
		
		if(abs(gl_TexCoord[0].t) <= 1.0) {
			r2 = 1.0 - gl_TexCoord[0].y * gl_TexCoord[0].y;
			r = sqrt(r2);
			s = gl_TexCoord[0].s + ds * r;

			if( s < 0.0 || s > 1.0) {
				//gl_FragColor = vec4(1.0, 0.0, 1.0, 1.0);
				discard;
			} else {
				//normal = gl_TexCoord[0].y * plane_offset + sqrt(r2)*plane_normal;
				colorv = get_frag_color(r2);

				gl_FragDepth = gl_FragCoord.z + 0.4 * dist * sqrt(r2);
				//gl_FragDepth = gl_FragCoord.z + dist * sqrt(r2);
			}
			// Do AO calculation here
			//local_normal = vec4(gl_TexCoord[0].t, r * ny, r * nz, 1.0);
			//local_normal = vec4(1.0, 0.0, 0.0, 1.0);
			//local_normal = vec4(gl_TexCoord[0].t * normalS + r * normalP, 1.0);
			//global_normal_tmp = transpose(gl_ModelViewMatrix) * local_normal;
			//global_normal = normalize(global_normal_tmp.xyz);

			// Doing lookup
			//vec2 tmp = normalize(vec2(dot(global_normal, normal_1), dot(global_normal, normal_2)));
			//univpos = vec3(tmp, 2.0 * s - 1.0);
			//if(univpos.x >= 0) {
			//  texpos = vec2( univpos.y / ( 2*(univpos.x + abs(univpos.y)) ) - 0.5, 2 * univpos.z);
			//} else {
			//  texpos = -1.0 * vec2( univpos.y / ( 2*(-1.0 * univpos.x + abs(univpos.y)) ) + 0.5, 2 * univpos.z);
			//}
			//texpos.s = gl_TexCoord[0].t;
			texpos = vec2(0.0, 4.0*s - 2.0);

			texpos = texpos * ao_strip_width + gl_TexCoord[0].pq;
			ao_col = texture1D(ao_func, texture2D(ambient_occlusion0, texpos).t);
			ao_col = colorv * (ao_col.x * (1.0 - ambLight) + ambLight);

			if( structural_abstraction < 0.125) {
			  ao_col = mix(vec4(0.0,0.0,0.0,1.0), ao_col, structural_abstraction * 8);
			} if (structural_abstraction < 0.25) {
			  ao_col = mix(ao_col, colorv, (structural_abstraction - 0.125) * 8);
			} else {
			  ao_col = colorv;
			}
			//gl_FragColor = ao_col;
			//gl_FragColor = vec4(vec3(structural_abstraction*4), 1.0);
			gl_FragColor = mix(colorv, ao_col, ao_contrib);

			//gl_FragColor = vec4(vec3(texture2D(ambient_occlusion0, texpos).g), 1.0);
			//gl_FragColor = vec4(dot(global_normal, normalize(direct)) * 0.5 + vec3(0.5),1.0);
			//gl_FragColor = vec4(global_normal * 0.5 + vec3(0.5),1.0);
			//gl_FragColor = vec4(global_normal * 0.5 + vec3(0.5),1.0);
			//gl_FragColor = vec4(texpos * 0.5 + vec2(0.5), 0.0, 1.0);
		} else {
			gl_FragColor = vec4(0.0);
			gl_FragDepth = gl_FragCoord.z + ((abs(gl_TexCoord[0].t) - 1.0) + 0.0)*0.0;
		}
	} else {
	  // Drawing endcap
		r2 = gl_TexCoord[0].s * gl_TexCoord[0].s + gl_TexCoord[0].t * gl_TexCoord[0].t;
		if(r2 < 1.0 && structural_abstraction > 0.25) {
			gl_FragColor = get_frag_color(1.0);
			if(illustrativeness > 0.5 && r2 > 1.0 - 0.5*(illustrativeness - 0.5)) {
				gl_FragColor = vec4(0.0);
				gl_FragColor.a = 1.0;
			}
			
			gl_FragDepth = gl_FragCoord.z - 0.0001;
		} else {
			discard;
		}
	}
}

