#version 120
uniform float depth_cueing;

uniform sampler2D ambient_occlusion0;
uniform sampler2D ambient_occlusion1;
uniform sampler1D ao_func;

uniform float ao_block_width;
uniform float ambLight;

varying float structural_abstraction;
varying float illustrativeness;
varying float radius;
varying float depth_top;
varying float id;
varying float halo_radius;
varying mat3 rotationMatrix;
varying float type;

// Pastel colors
varying vec3 pastel_light;
varying vec3 pastel_dark;

// Ambient Occlussion variables
varying float ao_fact;
varying float ao_contrib;

// Debug variables
varying float hint;
varying vec3 upVector;
varying vec3 camVector;

vec4 color(float r2) {
	float gradient;
	
	if (structural_abstraction < 0.6) {
		gradient = mix((1.0 - r2), 1.0, structural_abstraction / 0.6);
		return gl_Color * gradient;
	}
	else {
		return gl_Color;
	}
}

vec4 cell(float light_r, float r2) {
	float gradient;
	
	gradient = (1.0 - floor(light_r + 0.25)) * (1.0 - floor(r2 + 0.1));
	return vec4(mix(pastel_dark, pastel_light, gradient), 1.0);
	//cell = 0.4 * gradient + 0.6;
	//return cell * gl_Color;
}

vec4 grey(float light_r, float r2) {
	float grey;
	//float gradient;
	//float cell;
	vec4 col;

	//grey = (11.0 * gl_Color.r + 16.0 * gl_Color.g + 5.0 * gl_Color.b) / 32.0;
	//gradient = (1.0 - floor(light_r + 0.25)) * (1.0 - floor(r2 + 0.1));
	//cell = 0.4 * gradient + 0.6;
	//return vec4(cell * grey);
	col = cell(light_r, r2);
	grey = (0.30* col.r + 0.59 * col.g + 0.11 * col.b);
	return vec4(grey);
}

vec4 hatch(float r2) {
  vec4 ret;
  float theta, phi, z;
  float v1, v2;
  vec3 pos_local, pos_orig;
	float hatch;
  
  /* Create a border around the atom sphere */
  if (r2 > 0.8) {
    ret = vec4(0.0);
  }
  else {
  	z = sqrt(1.0 - gl_TexCoord[0].s * gl_TexCoord[0].s - gl_TexCoord[0].t * gl_TexCoord[0].t);
  	pos_local = vec3(gl_TexCoord[0].s, gl_TexCoord[0].t, z);
	pos_orig = rotationMatrix * pos_local;

	theta = asin(pos_orig.y);
	phi = atan(pos_orig.x,pos_orig.z);

	theta = fract(theta / 3.14 + 0.5);
	phi = fract(phi / 6.28);

	if (type < 1.0) {
		/* Non-(C/N/O) atom */
		//ret = vec4(0.0);
		//ret = vec4(ceil(fract((theta - 0.05*fract(10.0*phi)) * 13.0) - 0.3));
		ret = vec4(ceil(fract(phi * 7.0 - 13.0 * theta) - 0.2));
	}
	else if (type < 2.0) {
		/* C atom */
		ret = vec4(1.0);
	}
	else if (type < 3.0) {
		/* N atom */
		ret = vec4(ceil(fract(theta * 13.0) - 0.3));
		//ret = vec4(ceil(fract((phi * 17.0) + (theta * 13.0)) - 0.3));
	}
	else {
		/* O atom */
		hatch = ceil(fract(theta * 7.0) - 0.15) * ceil(fract(phi * 15.0 + 2.0 * theta) - 0.2);
		ret = vec4(ceil(hatch));
	}
	
	v1 = ceil(fract(theta * 10.0) - 0.8);
	v2 = ceil(fract(phi * 10.0) - 0.8);
  }
  
  return ret;
}

void main(void)
{
  vec3 normal;
  vec3 pos;
  vec2 coord;
  float r2;
  float depth;
  float depth_scale;
	float halo_r;
  float blend;
	vec4 col, ao_col, id_buff;
	int visible_count = 0;
	float detected_id;
	float light_r;
	vec2 light_c;
	float light_maxdist;

  coord = vec2(gl_TexCoord[0].x, gl_TexCoord[0].y);
  r2 = dot(gl_TexCoord[0].xy, gl_TexCoord[0].xy);
	halo_r = 1.0 + halo_radius;

  // Determine difference between nearest point on the sphere
  // and the center of the sphere.
  depth_scale = gl_FragCoord.z - depth_top;

  if (r2 > 1.0) {
		if (r2 < halo_r * halo_r) {
			gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
			//gl_FragColor.a = 1.0 - (sqrt(r2) - 1.0) / halo_r;
			gl_FragDepth = gl_FragCoord.z + (((r2 - 1.0) / (halo_r * halo_r - 1.0)) + 1.0) * depth_scale;
			return;
		}

    discard;
	}

  depth = sqrt(1.0 - r2);
	light_c = vec2(-0.2, 0.3);
	light_maxdist = length(light_c) + 1.0;
	light_r = length(gl_TexCoord[0].st - light_c) / light_maxdist;

  if (illustrativeness <= (1.0 / 3.0)) {
		blend = 3.0 * illustrativeness;
		col = mix(color(light_r), cell(light_r, r2), blend);
  }
  else if (illustrativeness <= (2.0 / 3.0)) {
    blend = 3.0 * (illustrativeness - (1.0 / 3.0));
		col = mix(cell(light_r, r2), grey(light_r, r2), blend);
	}
	else {
    blend = 3.0 * (illustrativeness - (2.0 / 3.0));
		col = mix(grey(light_r, r2), hatch(r2), blend);
  }
	
	//ao_col = texture1D(ao_func, ao_fact) * col;

	// Do AO lookup
  float z = sqrt(1.0 - gl_TexCoord[0].s * gl_TexCoord[0].s - gl_TexCoord[0].t * gl_TexCoord[0].t);
  vec3 pos_local = vec3(gl_TexCoord[0].s, gl_TexCoord[0].t, z);
	vec4 pos_orig_tmp = transpose(gl_ModelViewMatrix) * vec4(pos_local, 1.0);
	vec3 pos_orig = normalize(pos_orig_tmp.xyz);
	vec2 ao_tex_lookup;
	float d = abs(pos_orig.x) + abs(pos_orig.y) + abs(pos_orig.z);
	if(pos_orig.z <= 0.0) {
		ao_tex_lookup = vec2(pos_orig.x / d, pos_orig.y / d);
	} else {
		ao_tex_lookup = vec2(sign(pos_orig.x) * (1.0 - (abs(pos_orig.y) / d)), sign(pos_orig.y) * (1.0 - (abs(pos_orig.x) / d)));
	}
	ao_tex_lookup = ao_tex_lookup * ao_block_width + gl_TexCoord[0].pq;
	ao_col = texture1D(ao_func, texture2D(ambient_occlusion0, ao_tex_lookup).s);
	ao_col = mix(col * (ao_col.x * (1.0 - ambLight) + ambLight), col, structural_abstraction * 4);
	ao_col.a = 1.0;

	gl_FragColor = mix(col, ao_col, ao_contrib);
  gl_FragDepth = gl_FragCoord.z - (depth_scale * depth);
}


