#version 120
#extension GL_EXT_geometry_shader4 : enable

#define CLIPDIST 5.0
#define PROTSTART 1.5
#define PROTEND 4.0

uniform float mol_abstractionmax;
uniform float protein_radius;
uniform float depth_cueing;
uniform float attenuate;

varying in vec3 derivativeIn[];
varying in vec3 helix_directionIn[];
varying in float on_helixIn[];
varying in float stripe_coordIn[];

varying out float dist;
varying out vec3 plane_offset;
varying out vec3 plane_normal;
varying out vec4 color;
varying out float radius;

float mol_abstraction;
varying in float structural_abs[];
varying out float structural_abstraction;
varying in float illustrativenessIn[];
varying out float illustrativeness;

float halo_radius;
float line_attenuation;

// Pastel colors
varying in vec3 pastel_lightIn[];
varying out vec3 pastel_light;
varying in vec3 pastel_darkIn[];
varying out vec3 pastel_dark;

mat4 coords = mat4(0.0,-1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0,-1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0);

void stripOrientation(float widthIn, bool cylinder) {
	vec3 refVec[2];
	vec3 referenceVector[2];
  vec3 orig;
	vec4 pos[2];
	//vec3 plane_offset;
	vec3 offset[2];
	float width;
	vec4 texCoord;
	float scale;
	float dist[2];
	float attenuation;
	float distCam;

	width = 1.0 + halo_radius;
	scale = widthIn;

	pos[0] = gl_PositionIn[0];
	pos[1] = gl_PositionIn[1];
  orig = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
	refVec[0] = orig - vec3(pos[0]);
	refVec[1] = orig - vec3(pos[1]);
	dist[0] = (length(refVec[0]) - PROTSTART) / (PROTEND - PROTSTART);
	dist[1] = (length(refVec[1]) - PROTSTART) / (PROTEND - PROTSTART);
  referenceVector[0] = normalize(refVec[0]);
  referenceVector[1] = normalize(refVec[1]);
  offset[0] = normalize(cross(derivativeIn[0],referenceVector[0]));
  offset[1] = normalize(cross(derivativeIn[1],referenceVector[1]));

	distCam = 0.5 * (dist[0] + dist[1]);
	attenuation = mix(1.0, 1.5 * (1.0 - distCam) , line_attenuation);
	texCoord = vec4(-width, -1.0, 0.0, 0.0);
	gl_TexCoord[0] = texCoord;
	gl_FrontColor = gl_FrontColorIn[0];
	pastel_dark = pastel_darkIn[0];
	pastel_light = pastel_lightIn[0];
	gl_Position = gl_ModelViewProjectionMatrix * (pos[0] + vec4(-width * scale * attenuation * offset[0], 0.0));
	EmitVertex();

	texCoord.s = width;
	texCoord.t = 1.0;
	gl_TexCoord[0] = texCoord;
	gl_Position = gl_ModelViewProjectionMatrix * (pos[0] + vec4(width * scale * attenuation * offset[0], 0.0));
	EmitVertex();

	texCoord.s = -width;
	texCoord.t = -1.0;
	gl_TexCoord[0] = texCoord;
	gl_FrontColor = gl_FrontColorIn[1];
	pastel_dark = pastel_darkIn[1];
	pastel_light = pastel_lightIn[1];
	gl_Position = gl_ModelViewProjectionMatrix * (pos[1] + vec4(-width * scale * attenuation * offset[1], 0.0));
	EmitVertex();
  
	texCoord.s = width;
	texCoord.t = 1.0;
	gl_TexCoord[0] = texCoord;
	gl_Position = gl_ModelViewProjectionMatrix * (pos[1] + vec4(width * scale * attenuation * offset[1], 0.0));
	EmitVertex();
}

void helixOrientation(float scale, float blendIn, float on_helix) {
	vec4 pos[2];
	vec3 orig;
	vec3 refVec[2];
	float dist[2];
	float width;
	vec4 texCoord;
	float ribbon_width;
	float attenuation;
	vec3 dir[2];
	vec3 referenceVector[2];
	vec3 offset[2];
	vec4 helixColor = vec4(0.7, 0.0, 0.0, 1.0);
	vec3 helix_light = vec3(251.0/255.0, 106.0/255.0, 74.0/255.0);
	vec3 helix_dark  = vec3(103.0/255.0,   0.0/255.0, 13.0/255.0);
	float distCam;
	float blend[2];

	blend[0] = on_helixIn[0] * blendIn;
	blend[1] = on_helixIn[1] * blendIn;
	width = 1.0 + halo_radius;
	pos[0] = gl_PositionIn[0];
	pos[1] = gl_PositionIn[1];
	orig = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
	refVec[0] = orig - vec3(pos[0]);
	refVec[1] = orig - vec3(pos[1]);
	dist[0] = (length(refVec[0]) - PROTSTART) / (PROTEND - PROTSTART);
	dist[1] = (length(refVec[1]) - PROTSTART) / (PROTEND - PROTSTART);
	distCam = 0.5 * (dist[0] + dist[1]);
	attenuation = mix(1.0, 1.5 * (1.0 - distCam) , line_attenuation);
	ribbon_width = 0.5 * 4.0 * (structural_abstraction - 0.75) + 0.3;
	
	/* Interpolate from strip to ribbon */
	referenceVector[0] = normalize(refVec[0]);
  referenceVector[1] = normalize(refVec[1]);
  offset[0] = normalize(cross(derivativeIn[0],referenceVector[0]));
  offset[1] = normalize(cross(derivativeIn[1],referenceVector[1]));
	dir[0] = mix(scale * offset[0], ribbon_width * helix_directionIn[0], blend[0]);
	dir[1] = mix(scale * offset[1], ribbon_width * helix_directionIn[1], blend[1]);

	texCoord = vec4(-width, stripe_coordIn[0], floor(on_helix + 0.5), 0.0);
	gl_TexCoord[0] = texCoord;
	gl_FrontColor = mix(gl_FrontColorIn[0], helixColor, blend[0]);
	pastel_light = mix(pastel_lightIn[0], helix_light, blend[0]);
	pastel_dark = mix(pastel_darkIn[0], helix_dark, blend[0]);
	gl_Position = gl_ModelViewProjectionMatrix * (pos[0] + vec4(-width * attenuation * dir[0], 0.0));
	EmitVertex();

	texCoord.s = width;
	gl_TexCoord[0] = texCoord;
	gl_Position = gl_ModelViewProjectionMatrix * (pos[0] + vec4( width * attenuation * dir[0], 0.0));
	EmitVertex();

	texCoord.s = -width;
  texCoord.t = stripe_coordIn[1];
	gl_TexCoord[0] = texCoord;
	gl_FrontColor = mix(gl_FrontColorIn[1], helixColor, blend[1]);
	pastel_light = mix(pastel_lightIn[1], helix_light, blend[1]);
	pastel_dark = mix(pastel_darkIn[1], helix_dark, blend[1]);
	gl_Position = gl_ModelViewProjectionMatrix * (pos[1] + vec4(-width * attenuation * dir[1], 0.0));
	EmitVertex();

	texCoord.s = width;
	gl_TexCoord[0] = texCoord;
	gl_Position = gl_ModelViewProjectionMatrix * (pos[1] + vec4( width * attenuation * dir[1], 0.0));
	EmitVertex();
}

void main() {
	float tmp, radius, growth;

	illustrativeness = illustrativenessIn[0];
  structural_abstraction = structural_abs[0];

	if (structural_abstraction <= 0.6) {
		/* Cylinders */
		return;
  }

	if (depth_cueing < 0.5) {
		line_attenuation = 2.0 * depth_cueing * attenuate;
		halo_radius = 0.0;
	}
	else {
		line_attenuation = 1.0 * attenuate;
		halo_radius = mix(0.0, 2.0 * depth_cueing - 1.0, sqrt((structural_abstraction - 0.6) / 0.15));
	}

	if (structural_abstraction < 0.75) {
		/* AKA, were structural_abstraction <- [0.6, 0.75)
		/* Transition from linear to splines */
		radius = 0.1 * (1.5 / protein_radius) *
							mix(1.0, 2.0, (structural_abstraction - 0.6) / 0.15);
		stripOrientation(radius, false);
	}
	else {
		radius = 0.1 * (1.5 / protein_radius) * 2.0;
		/* Splines and helices */
		helixOrientation(radius, (structural_abstraction - 0.75) * 4.0, on_helixIn[0]);
	}
}
