#version 120
#extension GL_EXT_geometry_shader4 : enable

uniform float mol_abstractionmax;
uniform float protein_radius;
uniform float structural_abstraction;
uniform float depth_displacement;

varying in float atom_rank[];
varying in float atom_radius[];
varying out float depth_top;

float radius;
float mol_abstraction;

mat4 coords = mat4(-1.0, -1.0, 0.0, 0.0, -1.0, 1.0, 0.0, 0.0, 1.0, -1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0);

void calculate_abstraction() {
	float tmp, shrink;

	/* PRE: When this program is used: 0 <= structural_abs <= 0.6 */
	if (structural_abstraction < 0.25) {
		tmp = structural_abstraction * (4.0 * 0.9);
		shrink = 1.0 - 2.0 * structural_abstraction;
		radius = shrink * (atom_radius[0] / protein_radius) * (1.0 - tmp);
		mol_abstraction = mol_abstractionmax;
	}
	else {
		radius = (atom_radius[0] / protein_radius) * 0.05;
		tmp = 1.0/(0.6-0.25) * mol_abstractionmax * (structural_abstraction-0.25);
    mol_abstraction = mol_abstractionmax - tmp;
	}
}

vec4 determine_position(in vec4 vertex, inout vec4 coord) {
  vec4 pos;
  vec4 referenceVector;
  vec4 cam;
  vec4 displ;
  vec4 proj;

  // Figure out where the camera is in model coordinates
  cam = gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0);

  // Determine the displacement (in the direction of the eye) to the surface
  displ = radius * normalize(cam - vertex);

  // Transform the point on the surface to eye coordinates
  referenceVector = gl_ModelViewProjectionMatrix * (displ + vertex);

  // Calculate its (would be) fragment depth
  depth_top = (referenceVector.z / referenceVector.w) / 2.0 + 0.5;

  pos = gl_ModelViewMatrix * vertex;
  pos.x = pos.x + coord.s * (radius * 1.1);
  pos.y = pos.y + coord.t * (radius * 1.1);

  return pos;
}

void main() {
  vec4 coord;
  vec4 pos;
  
  calculate_abstraction();

	if (atom_rank[0] <= mol_abstraction) {
		for(int i = 0; i < 4; ++i) {
			coord = coords[i];
			gl_FrontColor = gl_FrontColorIn[0];
			//gl_FrontColor.a = mol_abstraction + atom_rank[0];

			pos = determine_position(gl_PositionIn[0], coord);
			gl_TexCoord[0] = coord;
			gl_Position = gl_ProjectionMatrix * pos;
			EmitVertex();
		}
	}
}