#version 120
attribute vec3 direction;
varying vec3 directionIn;

attribute vec2 ranks;
varying vec2 ranksIn;

varying vec3 pastel_light0In;
varying vec3 pastel_light1In;
varying vec3 pastel_dark0In;
varying vec3 pastel_dark1In;

/* Structural abstraction lookup variables */
attribute float abstraction_lookup_pos;
uniform sampler1D abstraction_lookup;
varying float structural_abs;
varying float illustrativenessIn;

uniform sampler2D colormap;
attribute vec2 element_coords;

attribute vec2 ao_tex_pos;
uniform float ao_strip_width;

varying float nyIn;
varying float nzIn;
varying vec3 normal_1In;
varying vec3 normal_2In;

varying vec3 normalSIn;
varying vec3 normalPIn;

void main() {
	vec4 x,y,z;
	vec3 plane_offset, plane_normal;

	gl_Position = gl_Vertex;
	gl_TexCoord[0] = gl_MultiTexCoord0;
	gl_TexCoord[0].pq = ao_tex_pos;
	directionIn = direction;
	ranksIn = ranks;
	
	/* Colors of first atom */
	gl_FrontColor = texture2D(colormap, vec2(0.0, element_coords[0]));
	pastel_light0In = texture2D(colormap, vec2(0.5, element_coords[0])).rgb;
	pastel_dark0In = texture2D(colormap, vec2(1.0, element_coords[0])).rgb;

	/* Colors of second atom */
  gl_FrontSecondaryColor = texture2D(colormap, vec2(0.0, element_coords[1]));
	pastel_light1In = texture2D(colormap, vec2(0.5, element_coords[1])).rgb;
	pastel_dark1In = texture2D(colormap, vec2(1.0, element_coords[1])).rgb;
	
	structural_abs = texture1D(abstraction_lookup, abstraction_lookup_pos).r;
	illustrativenessIn = texture1D(abstraction_lookup, abstraction_lookup_pos).b;

	x = normalize(transpose(gl_ModelViewMatrix) * vec4(1.0, 0.0, 0.0, 1.0));
	y = normalize(transpose(gl_ModelViewMatrix) * vec4(0.0, 1.0, 0.0, 1.0));
	z = normalize(transpose(gl_ModelViewMatrix) * vec4(0.0, 0.0, 1.0, 1.0));

	plane_offset = normalize(cross(z.xyz, direction));
	plane_normal = normalize(cross(direction, plane_offset));

	nyIn = dot(plane_normal, y.xyz);
	nzIn = dot(plane_normal, z.xyz);

	normalSIn = vec3(dot(plane_offset,x.xyz), dot(plane_offset, y.xyz), 0.0);
	normalPIn = vec3(dot(plane_normal,x.xyz), dot(plane_normal, y.xyz), dot(plane_normal, z.xyz));

	// Make a fixed local coordinate system on a cylinder
	normal_1In = vec3(direction.y, -1.0*direction.x,0);
	if(abs(direction.x) < 0.000000001 && abs(direction.y) < 0.000000001) {
	  normal_1In = vec3(1.0, 0.0, 0.0); 
	} else {
	  normal_1In = vec3(direction.y, -1.0*direction.x,0);
	}
	normal_1In = normalize(normal_1In);
	normal_2In = normalize(cross(normal_1In, direction));
}

